//*****************************************
// AVR ATtiny85 Dekatron(A-102) Spinner Program
// CPU Clock:8MHz
//   iq[Yf[^@UWlgɂj
//
// File Name:timer.c
//
//  2016/08/30 New(TAB4)  Taishi
//  2021/02/24 Modify     Taishi Z562S@Ήi_upX fJgj
//  2021/02/25 Modify     Taishi UqɕύX
//  2021/03/24 Modify     Taishi A-102 Ή
//*****************************************
#include	<avr/interrupt.h>
#include	<avr/io.h>


#include	"tiny85_a102_spinner.h"
#include	"timer.h"


volatile unsigned int	wait_i;
volatile unsigned char	wait_us;
unsigned int			tim_1ms_pendulum;				// UqpPrPʃ^C}[
unsigned int			tim_1ms_inchworm;				// ڎ撎pPrPʃ^C}[
unsigned int			tim_10ms_move_speed;			// 쑬xpPOrPʃ^C}[
unsigned char			tim_10ms_led;					// kdcpPOrPʃ^C}[
unsigned char			tim_10ms_sw;					// rvhsbgpPOrPʃ^C}[
unsigned char			tim_10ms_swr_length;			// rvhsbgpԂPOrPʃ^C}[
unsigned char			tim_10ms_swr_continue;			// rvhsbgpAPOrPʃ^C}[



// **********************************************************
// ************************ Timer 0 *************************
// **********************************************************
// 1mS Interrupt
// #define		COUNT_1MS	150						// 1mS at 9.6MHz/64
#define		COUNT_1MS	((CPU_CLOCK/64)/1000)		// Countor Valur of 1mS
#define		D_TCCR0A	((0 << COM0A1) | ( 0 << COM0A0)	| (0 << COM0B1) | ( 0 << COM0B0) | (1 << WGM01) | (0 << WGM00)) 	// A Comper Match, Dont Timer0 Signal
#define		D_TCCR0B	((0 << FOC0A) | (0 << FOC0B) | (0 << WGM02) | (0 << CS02) | (1 << CS01) | (1 << CS00))				// Timer0 Start (Clock/64)

// ************************************
// Delay 2uS
//  t: x2uS (max 255)
//  CPU Clock:9.6MHz 
// ************************************
void delay_2us(unsigned char t)
{
	while(t-- != 0){
		wait_us++;				// 1uS Delay??
		wait_us++;				// 1uS Delay??
		wait_us++;				// 1uS Delay??
	}
}
// ************************************
// Delay 50uS
//  CPU Clock:9.6MHz
// ************************************
void	delay_50us(void)
{
	delay_2us(50/2);
}
//***************************
// Wait
//  tm : Wait Time(ms) max. 8000
//  CPU Clock:9.6MHz
//***************************
void	delay_ms(int tm)
{

	tm *= 4;
	for(wait_i = 0; wait_i < tm; wait_i++){
		delay_2us(250/2);
	}
}
//***************************
// Timer0 Initial
// CPU Clock:9.6MHz
//  Set to 1mS Interrupt
//***************************
void	timer0_init(void)
{
 	TCCR0B = 0;						// Timer0 Stop
	TCNT0 = 0;						// Timer0 Count Set to ZERO
	TCCR0A = D_TCCR0A;				// A Comper Match, Dont Timer0 Signal
	OCR0A  = COUNT_1MS;				// 1mS
	TIMSK  |= (1 << OCIE0A);		// A match Interrupt Enable
 	TCCR0B = D_TCCR0B;				// Timer0 Start (Clock/64)

	tim_10ms_sw = 0;				// rvhsbgpPOrPʃ^C}[
	tim_10ms_swr_length = 0;		// rvhsbgpԂPOrPʃ^C}[
	tim_10ms_swr_continue = 0;		// rvhsbgpAPOrPʃ^C}[
	tim_10ms_led = 0;				// kdcpPOrPʃ^C}[

	tim_1ms_pendulum = 0;			// UqpPrPʃ^C}[
	tim_1ms_inchworm = 0;			// ڎ撎pPrPʃ^C}[
	tim_10ms_move_speed = 0;		// 쑬xpPOrPʃ^C}[
}
//***************************
// Timer0 Comper Match A Interrupt
//
// CPU Clock:9.6MHz(1mS Interrupt)
//***************************
ISR	(TIM0_COMPA_vect)
{
	static unsigned char	count_10ms = 0;

	count_10ms++;
	if(10 <= count_10ms){
		count_10ms = 0;							// 1mS x 10 = 10mS
		if(tim_10ms_sw != 0){
			tim_10ms_sw--;						// rvhsbgpPOrPʃ^C}[@|P
		}

		if(tim_10ms_swr_continue != 0){
			tim_10ms_swr_continue--;			// rvhsbgpAPOrPʃ^C}[@|P
		}

		if(tim_10ms_swr_length != 0){
			tim_10ms_swr_length--;				// rvhsbgpԂPOrPʃ^C}[@|P
		}

		if(tim_10ms_led != 0){
			tim_10ms_led--;						// kdcpPOrPʃ^C}[@|P
		}

		if(tim_10ms_move_speed != 0){
			tim_10ms_move_speed--;				// 쑬xpPOrPʃ^C}[ |P
		}
	}

	if(tim_1ms_pendulum != 0){
		tim_1ms_pendulum--;						// UqpPrPʃ^C}[@|P
	}

	if(tim_1ms_inchworm != 0){
		tim_1ms_inchworm--;						// ڎ撎pPrPʃ^C}[
	}

}
